import pytest
from pathlib import Path
import numpy as np

from dvdp.utils.data import DataSaver


def test_create_and_load_chunks(tmpdir):
    data_saver = DataSaver(
        Path(tmpdir) / 'test',
        ['col1', 'col2'],
        chunk_size=4,
    )
    expected = {'col1': [], 'col2': []}
    for i in range(8):
        new = {'col1': i+1, 'col2': i}
        data_saver.add_data(new)
        expected['col1'].append(new['col1'])
        expected['col2'].append(new['col2'])

    result = DataSaver.load_data(Path(tmpdir) / 'test_000.gz')
    assert result == expected


def test_with_image(tmpdir):
    data_saver = DataSaver(
        Path(tmpdir) / 'test',
        ['col1', 'col2'],
        chunk_size=4,
    )
    expected = {'col1': [], 'col2': []}
    for i in range(8):
        image = np.full((4, 4), i)
        new = {'col1': image, 'col2': image}
        data_saver.add_data(new)
        expected['col1'].append(new['col1'])
        expected['col2'].append(new['col2'])

    result = DataSaver.load_data(Path(tmpdir) / 'test_000.gz')
    assert len(result) == len(expected)
    for images_rs, images_exp in zip(result.values(), expected.values()):
        assert len(images_rs) == len(images_exp)
        for image_rs, image_exp in zip(images_rs, images_exp):
            assert np.all(image_rs == image_exp)




from pathlib import Path
from typing import Dict, List, Any

import pandas as pd


class DataSaver:
    def __init__(self, name: Path, column_names: List[str], chunk_size=200):
        """
        Create a datasaver that saves data when chunksize has been reached.
        :param name: location and name of file chunk. e.g. /tmp/test will
        product /tmp/test_00x.gz chunks.
        :param column_names: column names to allow for records containing
        multiple values.
        :param chunk_size: chunk size in records that triggers a save.
        """
        self.__name = str(name)
        self.__chunk_size = chunk_size
        self.__identifiers = column_names
        self.__data_frame = None
        self.__reset()
        self.__chunk_idx = 0

    def __reset(self):
        self.__data_frame = pd.DataFrame({
            idn: [] for idn in self.__identifiers
        })

    def __save_maybe(self):
        if len(self.__data_frame) % self.__chunk_size != 0:
            return
        self.__save()

    def __save(self):
        chunk_filename = f'{self.__name}_{self.__chunk_idx:03d}.gz'
        print(f'Saving chunk {self.__chunk_idx} to {chunk_filename}...')
        self.__chunk_idx += 1
        self.__data_frame.to_pickle(chunk_filename, compression='gzip')
        print('Finished saving chunk file.')
        self.__reset()

    def add_data(self, data: Dict[str, List[Any]]):
        self.__data_frame = self.__data_frame.append(data, ignore_index=True)
        self.__save_maybe()

    @classmethod
    def load_data(cls, first_file):
        """Load data from chunks."""
        first_file = Path(first_file)
        dir = Path(first_file).parent
        ext = first_file.suffix

        file_idx = -1
        files = []
        while True:
            file_idx += 1
            file_name = str(first_file.stem).replace(
                '_000',
                f'_{file_idx:03}{ext}',
            )
            file_cur = dir / file_name
            if not file_cur.exists():
                break
            files.append(file_cur)

        data = pd.read_pickle(str(files[0]))
        for file in files[1:]:
            data = data.append(pd.read_pickle(str(file)))
        return {
            col: [x for x in data[col]]
            for col in data
        }

    def __del__(self):
        self.__save()



import argparse
import asyncio
import logging

from dvdp.ha_mqtt.client import MQTTClient
from dvdp.ha_mqtt.light import HAMQTTLight


def on_light_state_change(state):
    logging.info(f'State of light changed to {state}')


def light(args, client):
    initial_state = args.state
    logging.info(f'Initial state of light: {initial_state}')
    ha_mqtt_light = HAMQTTLight(
        args.name,
        client,
        state_change_callback=on_light_state_change,
    )
    loop = asyncio.get_event_loop()
    task = loop.create_task(ha_mqtt_light.start())
    loop.run_until_complete(task)


def switch(args, client):
    logging.warning('Switch is not available for testing.')


def main():
    parser = argparse.ArgumentParser('Test mqtt devices.')
    parser.add_argument('name', help='Name of device.')
    parser.add_argument('brokerip', help='IP of broker.')
    parser.add_argument('--username', '-u', help='MQTT username.')
    parser.add_argument('--password', '-p', help='MQTT password.')
    parser.add_argument(
        '-v',
        '--verbose',
        help='Verbose logging',
        default=False,
        action='store_true',
    )
    subparsers = parser.add_subparsers(help='Choose a device to test')
    parser_light = subparsers.add_parser('light', help='Simulate Light')
    parser_light.add_argument(
        '--state',
        '-s',
        default=False,
        type=bool,
        help='Initial state of light.'
    )
    parser_light.set_defaults(func=light)
    parser_switch = subparsers.add_parser('switch', help='Simulate Switch')
    parser_switch.set_defaults(func=switch)
    args = parser.parse_args()
    if args.verbose:
        level = logging.DEBUG
    else:
        level = logging.INFO
    logging.basicConfig(
        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
        level=level,
    )

    client = MQTTClient(
        args.brokerip,
        'test_client',
        args.username,
        args.password,
    )

    args.func(args, client)


if __name__ == '__main__':
    main()


from setuptools import setup
from pathlib import Path

this_dir = Path(__file__).parent

with open(this_dir / 'requirements.txt') as file:
    requirements = '\n'.join(file.readlines())

with open(this_dir / 'README.md') as file:
    long_description = file.read()

version = '0.0.1'
package_name = 'utils'
top_ns = 'dvdp'

setup(
    name=f'{top_ns}.{package_name}',
    version=version,
    packages=[f'{top_ns}.{package_name}'],
    download_url=f'https://github.com/davidvdp/{package_name}/archive/v'
                 f'{version}.tar.gz',
    url=f'https://github.com/davidvdp/{package_name}',
    author='David van der Pol',
    author_email='david@davidvanderpol.com',
    license='MIT',
    description='Python utilities.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=requirements,
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
    ],
    keywords=[
        'Utils'
    ],
)